﻿
using System;
using System.IO.Ports;
using System.Windows.Forms;
using System.Threading;
using System.Collections;

namespace XMC_EKG
{
	/// <summary>
	/// EKG Klasse - Verwaltet die Kommunikation und die Register der EKG Hardware
	/// </summary>
	public class EKG
	{
		
		
		//_________________________Event - Delegate___________________//
		public event NewDataHandler NewDataEvent;
		public delegate void NewDataHandler(int[] data);
		
		
		//_________________________Felder___________________//
		
		private bool 	IsConnected_field;
		private bool 	IsSampling_field;
		private byte[]	LastSample_field = new byte[27];
		private bool	AutoLeadOffEnabled_field;
		
		
		
		//_________________________Properties___________________//
		
		public bool 	IsConnected
		{
			get { return IsConnected_field; }
		}
		
		public bool 	IsSampling
		{
			get { return IsSampling_field; }
		}
		
		
		public int 	BaudRate
		{
			get { return serialPort1.BaudRate; }
			set { serialPort1.BaudRate = value; }
		}
		
		public string PortName
		{
			get{ return serialPort1.PortName; }
			set{ serialPort1.PortName = value; }			
		}
		
		
		public byte[]	LastSample
		{ 
			get { return LastSample_field; }
		}
		
		
		public bool		AutoLeadOffEnabled
		{
			get { return AutoLeadOffEnabled_field; }
			set { AutoLeadOffEnabled_field = value; }
		}
		
		

		
		public int		LOFFP;
		

		public int		LOFFN;
		
		
	
		
		
	
		
		
		
		
		//_________________________Objekte Instaziieren___________________//
		
		private SerialPort serialPort1 = new SerialPort();	//Seriellen Port für die Kommunikation zum EKG hinzufügen
		public Queue Daten_FIFO = new Queue();				//Neuen FIFO Speicher für die speicherung der Samples
		
		
		
		//_________________________Standartkonstruktor___________________//
		public EKG()
		{	
			
		}
		
		
		
		//_________________________Methoden___________________//

		public void Connect()
		{		
			try
			{
				serialPort1.Open();
				serialPort1.DiscardInBuffer();
				SendByte(1);
			}
			catch(Exception ex)
			{
				serialPort1.Close();
				IsConnected_field = false;
				throw new Exception (ex.Message);
			}
			
			try
			{
				WaitForACK();
			}
			catch(Exception ex)
			{
				serialPort1.Close();
				IsConnected_field = false;
				throw new Exception (ex.Message + "\r\nSerieller Port weiterhin geschlossen!");
			}
			
			
			int ID;
			try
			{
				SendByte(4);
				ID = WaitForByte();
			}
			catch(Exception ex)
			{
				SendByte(2);
				serialPort1.Close();
				IsConnected_field = false;
				throw new Exception (ex.Message + "\r\nSerieller Port weiterhin geschlossen!");
			}
			
			if(ID != 0x92)
			{
				SendByte(2);
				serialPort1.Close();
				IsConnected_field = false;
				throw new Exception ("Falsche Hardware ID!\r\nEingelesene ID: " + ID + "\r\nErwartete ID: 146");
			}
			
			
			IsConnected_field = true;
		}
		
		public void Disconnect()
		{
			IsConnected_field = false;
			serialPort1.DiscardInBuffer();
			
			try
			{
				
				SendByte(2);
			}
			catch(Exception ex)
			{
				serialPort1.Close();
				IsConnected_field = false;
				throw new Exception (ex.Message + "\r\nSerieller Port wurde geschlossen! Evtl. hat die EKG Hardware das trennen der Verbindung nicht mitbekommen. Bitte EKG neu anschließen");		
			}
			
			try
			{
				WaitForACK();
			}
			catch(Exception ex)
			{
				serialPort1.Close();
				IsConnected_field = false;
				throw new Exception (ex.Message + "\r\nSerieller Port wurde geschlossen! Evtl. hat die EKG Hardware das trennen der Verbindung nicht mitbekommen. Bitte EKG neu anschließen");
			}
			
			IsSampling_field = false;
			serialPort1.Close();
			IsConnected_field = false;
		}
		
		
		private void PollSample ()
		{			
			Daten_FIFO.Clear();
			while(IsSampling == true)
			{
				if(serialPort1.BytesToRead > 0)
				{
					Daten_FIFO.Enqueue(serialPort1.ReadByte());
				}
				
				if(Daten_FIFO.Count >= 27)
				{
					//Thread Calculate_Thread = new Thread(CalculateSample);
					//Calculate_Thread.IsBackground = true;
					//Calculate_Thread.Start();
					int[] data = new int[27];
					for (int i = 0; i <= 26; i++)
        			{
           			 	data[i] = (int)Daten_FIFO.Dequeue();
        			}
					CalculateSample(data);
				}
				
			}
		}
		
		
		private void CalculateSample(int[] data)
		{
			
			// lead[0] = lead 2
			// lead[1] = lead 3
			// lead[2] = V1
			// lead[3] = V2
			// lead[4] = V3
			// lead[5] = V4
			// lead[6] = V5
			// lead[7] = V6
			
			int[] lead = new int[8];
			
			//____________LEAD2____________//
			lead[0] = (data[3] * 65535)+(data[4] * 256)+data[5];
			
			if(lead[0] >= 0x800000)
			{
				lead[0] -= 0x1000000;
			}
			lead[0] *= -1;
			
			
			//____________LEAD3____________//
			lead[1] = (data[6] * 65535)+(data[7] * 256)+data[8];
			
			if(lead[1] >= 0x800000)
			{
				lead[1] -= 0x1000000;
			}
			lead[1] *= -1;
			
			//____________V1____________//
			lead[2] = (data[9] * 65535)+(data[10] * 256)+data[11];
			
			if(lead[2] >= 0x800000)
			{
				lead[2] -= 0x1000000;
			}
			lead[2] *= -1;
			
			//____________V2____________//
			lead[3] = (data[12] * 65535)+(data[13] * 256)+data[14];
			
			if(lead[3] >= 0x800000)
			{
				lead[3] -= 0x1000000;
			}
			lead[3] *= -1;
			
			//____________V3____________//
			lead[4] = (data[15] * 65535)+(data[16] * 256)+data[17];
			
			if(lead[4] >= 0x800000)
			{
				lead[4] -= 0x1000000;
			}
			lead[4] *= -1;
			                                                
			//____________V4____________//
			lead[5] = (data[18] * 65535)+(data[19] * 256)+data[20];
			
			if(lead[5] >= 0x800000)
			{
				lead[5] -= 0x1000000;
			}
			lead[5] *= -1;
			
			//____________V5____________//
			lead[6] = (data[21] * 65535)+(data[22] * 256)+data[23];
			
			if(lead[6] >= 0x800000)
			{
				lead[6] -= 0x1000000;
			}
			lead[6] *= -1;
			
			//____________V6____________//
			lead[7] = (data[24] * 65535)+(data[25] * 256)+data[26];
			
			if(lead[7] >= 0x800000)
			{
				lead[7] -= 0x1000000;
			}
			lead[7] *= -1;


			
			
			NewDataEvent(lead);
		}
		
		private void WaitForACK()
		{
			Thread.Sleep(30);
			if(serialPort1.BytesToRead != 0)
			{
				if(serialPort1.ReadByte() == 6)
				{
					return;
				}
				else
				{
					throw new Exception ("Falsches ACK!\r\nNach einem Befehl wurde ein falsches ACK-Byte empfangen!");
				}
			}
			else
			{
				throw new Exception ("Acknowledgement Read Timeout!\r\nNach einem Befehl wurde kein ACK-Byte empfangen!");
			}			
		}
		
		private int WaitForByte()
		{
			Thread.Sleep(30);
			if(serialPort1.BytesToRead != 0)
			{
				return(serialPort1.ReadByte());
			}
			else
			{
				throw new Exception ("Serial Read Timeout!\r\nNach einer Anfrage wurde kein Byte empfangen!");
			}			
		}

		private void SendByte(byte wert)
		{
			byte[] jaa = new byte[1] {wert};		//Neue Byte Variable
			
			serialPort1.Write(jaa, 0, 1);		//Sende Byte Variable
		}	
		
		public void StartSampling()
		{
			try
			{
				serialPort1.DiscardInBuffer();
				SendByte(5);
				WaitForACK();
				IsSampling_field = true;
				Thread Sample_Thread = new Thread(PollSample);
				Sample_Thread.IsBackground = true;
				Sample_Thread.Start();
			}
			catch(Exception ex)
			{
				SendByte(6);
				IsSampling_field = false;
				throw new Exception (ex.Message + "\r\nSampling wurde nicht gestartet");
			}
				
		}
		
		
		public void StopSampling()
		{
			try
			{	
				IsSampling_field = false;				
				SendByte(6);
				serialPort1.DiscardInBuffer();
				
				//Sample_Thread.Abort();
			}
			catch(Exception ex)
			{
				SendByte(6);
				IsSampling_field = false;
				throw new Exception (ex.Message + "\r\nSampling wurde nicht gestartet");
			}
				
		}
	}
}
